# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_documents', '0002_document_tags'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='parent_category',
            field=models.ForeignKey(to='glitter_documents.Category', blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import taggit.managers


class Migration(migrations.Migration):

    dependencies = [
        ('taggit', '0002_auto_20150616_2121'),
        ('glitter_documents', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='document',
            name='tags',
            field=taggit.managers.TaggableManager(through='taggit.TaggedItem', verbose_name='Tags', to='taggit.Tag', help_text='A comma-separated list of tags.', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os

from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from glitter.mixins import GlitterMixin
from glitter.models import BaseBlock
from PIL import Image
from taggit.managers import TaggableManager


@python_2_unicode_compatible
class Format(models.Model):
    title = models.CharField(max_length=100, db_index=True)

    class Meta:
        ordering = ('title', )

    def __str__(self):
        return self.title


@python_2_unicode_compatible
class Category(models.Model):
    title = models.CharField(max_length=100, db_index=True)
    slug = models.SlugField(max_length=100, unique=True)
    parent_category = models.ForeignKey('self', null=True, blank=True)

    class Meta:
        verbose_name_plural = 'categories'
        ordering = ('title', )

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('glitter-documents:category-list', args=(self.slug, ))


@python_2_unicode_compatible
class Document(GlitterMixin):
    title = models.CharField(max_length=100, db_index=True)
    slug = models.SlugField(max_length=100, unique=True)
    category = models.ForeignKey(Category)
    document = models.FileField(max_length=200, upload_to='documents/document/%Y/%m')
    valid_image = models.BooleanField(default=False, editable=False)
    author = models.CharField(blank=True, max_length=32)
    file_size = models.PositiveIntegerField(default=0, editable=False)
    document_format = models.ForeignKey(Format)
    summary = models.TextField(blank=True)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)
    updated_at = models.DateTimeField(auto_now=True, db_index=True)

    tags = TaggableManager(blank=True)

    class Meta(GlitterMixin.Meta):
        get_latest_by = 'created_at'
        ordering = ('-created_at', )

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('glitter-documents:detail', args=(self.slug, ))

    def save(self, *args, **kwargs):
        # Avoid doing file size requests constantly
        self.file_size = self.document.size

        # See if it's a valid image, so we can show a thumbnail for it
        try:
            Image.open(self.document).verify()
            self.valid_image = True
        except Exception:
            self.valid_image = False

        super(Document, self).save(*args, **kwargs)

    def get_file_extension(self):
        return os.path.splitext(self.document.name)[1]

    def get_file_name(self):
        return os.path.basename(self.document.name)


class LatestDocumentsBlock(BaseBlock):
    category = models.ForeignKey(Category, null=True, blank=True)

    class Meta:
        verbose_name = 'latest documents'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import template
from django.db.models import Q

from ..models import Category, Document

register = template.Library()


@register.assignment_tag
def get_latest_documents(count=5, category=None):
    """ Accepts category or category slug. """
    document_list = Document.objects.published()

    # If object is given as a slug, fetch the actual object or set category
    # to None so it is not used for filtering.
    if category is not None:
        if not isinstance(category, Category):
            try:
                category = Category.objects.get(slug=category)
            except Category.DoesNotExist:
                category = None

    # Optional filter by category, can be either a sub or parent category
    if category:
        query = Q(pk=category.pk) | Q(parent_category=category)
        categories = Category.objects.filter(query)
        document_list = document_list.filter(category__in=categories)

    return document_list[:count]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.apps import apps
from django.conf import settings
from django.contrib import admin
from django.utils.html import mark_safe

from glitter import block_admin
from glitter.admin import GlitterAdminMixin

from .models import Category, Document, Format, LatestDocumentsBlock


@admin.register(Format)
class FormatAdmin(admin.ModelAdmin):
    search_fields = ('title', )


@admin.register(Category)
class CategoryAdmin(admin.ModelAdmin):
    search_fields = ('title', )
    prepopulated_fields = {'slug': ('title', 'parent_category')}

    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        qs = form.base_fields['parent_category'].queryset
        # To restrict to only 1 level of dependency we do the following:
        # - Only show categories that have no parents themselves.
        qs = qs.filter(parent_category__isnull=True)

        if obj:
            if obj.category_set.all().exists():
                # If this category is a parent, it can not have a parent.
                qs = Category.objects.none()
            else:
                # Exclude itself, you can't be your own parent.
                qs = qs.exclude(pk=obj.pk)

        form.base_fields['parent_category'].queryset = qs

        return form


@admin.register(Document)
class DocumentAdmin(GlitterAdminMixin, admin.ModelAdmin):
    date_hierarchy = 'created_at'
    list_display = (
        'title', 'category', 'document_format', 'file_size', 'view_on_site_link', 'is_published'
    )
    list_filter = ('category', )
    prepopulated_fields = {'slug': ('title', )}

    def get_inline_instances(self, request, obj=None):
        if apps.is_installed('glitter.reminders'):
            # Import here to prevent migrations on the glitter level.
            from glitter.reminders.admin import ReminderInline
            if ReminderInline not in self.inlines:
                self.inlines.append(ReminderInline)
        return super(DocumentAdmin, self).get_inline_instances(request, obj)

    def get_fieldsets(self, request, obj=None):
        advanced_options = ['tags', 'slug']
        if not getattr(settings, 'GLITTER_DOCUMENTS_TAGS', False):
            advanced_options.remove('tags')

        fieldsets = ((
            'Document', {
                'fields': (
                    'title', 'category', 'author', 'document', 'document_format', 'summary',
                )
            }
        ), ('Advanced options', {'fields': advanced_options}), )
        return fieldsets

    def view_on_site_link(self, obj):
        link_html = '<a href="{url}">View document</a>'.format(url=obj.get_absolute_url())
        return mark_safe(link_html)

    view_on_site_link.short_description = 'View document'


block_admin.site.register(LatestDocumentsBlock)
block_admin.site.register_block(LatestDocumentsBlock, 'App Blocks')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('title', models.CharField(max_length=100, db_index=True)),
                ('slug', models.SlugField(max_length=100, unique=True)),
            ],
            options={
                'ordering': ('title',),
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('published', models.BooleanField(default=True, db_index=True)),
                ('title', models.CharField(max_length=100, db_index=True)),
                ('slug', models.SlugField(max_length=100, unique=True)),
                ('document', models.FileField(max_length=200, upload_to='documents/document/%Y/%m')),
                ('valid_image', models.BooleanField(editable=False, default=False)),
                ('author', models.CharField(max_length=32, blank=True)),
                ('file_size', models.PositiveIntegerField(editable=False, default=0)),
                ('summary', models.TextField(blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('updated_at', models.DateTimeField(auto_now=True, db_index=True)),
                ('category', models.ForeignKey(to='glitter_documents.Category')),
                ('current_version', models.ForeignKey(editable=False, blank=True, to='glitter.Version', null=True)),
            ],
            options={
                'get_latest_by': 'created_at',
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'edit', 'publish'),
                'ordering': ('-created_at',),
            },
        ),
        migrations.CreateModel(
            name='Format',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('title', models.CharField(max_length=100, db_index=True)),
            ],
            options={
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='LatestDocumentsBlock',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('category', models.ForeignKey(blank=True, to='glitter_documents.Category', null=True)),
                ('content_block', models.ForeignKey(editable=False, null=True, to='glitter.ContentBlock')),
            ],
            options={
                'verbose_name': 'latest documents',
            },
        ),
        migrations.AddField(
            model_name='document',
            name='document_format',
            field=models.ForeignKey(to='glitter_documents.Format'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.db.models import Q
from django.shortcuts import get_object_or_404
from django.views.generic import DetailView, ListView

from glitter.mixins import GlitterDetailMixin

from .mixins import DocumentMixin
from .models import Category, Document


class DocumentListView(DocumentMixin, ListView):
    paginate_by = 10
    queryset = Document.objects.published()


class DocumentDetailView(DocumentMixin, GlitterDetailMixin, DetailView):

    def get_context_data(self, **kwargs):
        context = super(DocumentDetailView, self).get_context_data(**kwargs)
        context['current_category'] = self.object.category
        return context


class CategoryDocumentListView(DocumentMixin, ListView):
    template_name_suffix = '_category_list'
    paginate_by = 10
    queryset = Document.objects.published()

    def get_queryset(self):
        qs = super(CategoryDocumentListView, self).get_queryset()
        self.category = get_object_or_404(Category, slug=self.kwargs['slug'])

        query = Q(pk=self.category.pk) | Q(parent_category=self.category)
        categories = Category.objects.filter(query)

        return qs.filter(category__in=categories)

    def get_context_data(self, **kwargs):
        context = super(CategoryDocumentListView, self).get_context_data(**kwargs)
        context['current_category'] = self.category
        return context



# -*- coding: utf-8 -*-

from .models import Category, Document


class DocumentMixin(object):
    model = Document

    def get_context_data(self, **kwargs):
        context = super(DocumentMixin, self).get_context_data(**kwargs)
        context['documents_categories'] = True
        categories = Category.objects.filter(parent_category__isnull=True)
        context['categories'] = categories
        return context



# -*- coding: utf-8 -*-

from haystack import indexes

from .models import Document


class DocumentIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(document=True, use_template=True)

    def get_model(self):
        return Document

    def index_queryset(self, using=None):
        return self.get_model().objects.published().select_related()



# -*- coding: utf-8 -*-

from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^$', views.DocumentListView.as_view(), name='list'),
    url(r'^(?P<slug>[\w-]+)/$', views.DocumentDetailView.as_view(), name='detail'),
    url(
        r'^category/(?P<slug>[-\w]+)/$',
        views.CategoryDocumentListView.as_view(),
        name='category-list'
    ),
]
